﻿using ProjectSeaplane.Drawings;
using ProjectSeaplane.Entities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectSeaplane;

public partial class FormPlaneConfig : Form
{
    private DrawingPlane _plane;
    private event Action<DrawingPlane>? PlaneDelegate;
    public FormPlaneConfig()
    {
        InitializeComponent();
        panelRed.MouseDown += panel_MouseDown;
        panelGreen.MouseDown += panel_MouseDown;
        panelBlue.MouseDown += panel_MouseDown;
        panelYellow.MouseDown += panel_MouseDown;
        panelWhite.MouseDown += panel_MouseDown;
        panelGray.MouseDown += panel_MouseDown;
        panelBlack.MouseDown += panel_MouseDown;
        panelPurple.MouseDown += panel_MouseDown;
        // TODO buttonCancel.Click привязать анонимный метод через lambda с закрытием формы
        buttonCancel.Click += (sender, e) => Close();
    }

    public void AddEvent(Action<DrawingPlane> planeDelegate)
    {
        if (PlaneDelegate == null)
        {
            PlaneDelegate = planeDelegate;
        }
        else
        {
            PlaneDelegate += planeDelegate;
        }
    }

    private void buttonAdd_Click(object sender, EventArgs e)
    {
        if (_plane != null)
        {
            PlaneDelegate?.Invoke(_plane);
            Close();
        }
    }

    private void labelObject_MouseDown(object sender, MouseEventArgs e)
    {
        (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
    }

    private void panelObject_DragEnter(object sender, DragEventArgs e)
    {
        e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
    }
    private void DrawObject()
    {
        Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _plane?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
        _plane?.SetPosition(5, 5);
        _plane?.DrawTransport(gr);
        pictureBoxObject.Image = bmp;
    }

    private void panelObject_DragDrop(object sender, DragEventArgs e)
    {
        switch (e.Data?.GetData(DataFormats.Text)?.ToString())
        {
            case "labelSimpleObject":
                _plane = new DrawingPlane((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White);
                break;
            case "labelModifiedObject":
                _plane = new DrawingSeaplane((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value,
                            Color.White, Color.Black, checkBoxFloats.Checked, checkBoxInflatableBoat.Checked);
                break;
        }
        DrawObject();
    }

    private void labelBodyColor_DragEnter(object sender, DragEventArgs e)
    {
        if (e.Data.GetDataPresent(typeof(Color)))
        {
            e.Effect = DragDropEffects.Copy;
        }
        else
        {
            e.Effect = DragDropEffects.None;
        }
    }

    private void labelBodyColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_plane != null)
        {
            _plane.EntityPlane.setBodyColor((Color)e.Data.GetData(typeof(Color)));
            DrawObject();
        }
    }

    private void labelAdditionalColor_DragEnter(object sender, DragEventArgs e)
    {
        if (_plane is DrawingSeaplane)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
    }

    private void labelAdditionalColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_plane.EntityPlane is EntitySeaplane _seaplane)
        {
            _seaplane.setAdditionalColor((Color)e.Data.GetData(typeof(Color)));
        }
        DrawObject();
    }

    private void panel_MouseDown(object sender, MouseEventArgs e)
    {
        (sender as Control).DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);
    }

    private void panelRed_Paint(object sender, PaintEventArgs e)
    {

    }
}
